@@ -352,7 +352,7 @@ static int _daemon_write(struct dm_event_fifos *fifos,
 int daemon_talk(struct dm_event_fifos *fifos,
 		struct dm_event_daemon_message *msg, int cmd,
 		const char *dso_name, const char *dev_name,
-		enum dm_event_mask evmask, uint32_t timeout)
+		unsigned evmask, uint32_t timeout)
 {
 	int msg_size;
 	memset(msg, 0, sizeof(*msg));
@@ -446,6 +446,7 @@ static int _check_for_usable_fifos(char *dmeventd_path, struct dm_event_fifos *f
 			log_error("%s is no longer a secure root-owned fifo with mode 0600.", fifos->client_path);
 			if (close(fifos->client))
 				log_sys_debug("close", fifos->client_path);
+			fifos->client = -1;
 			return 0;
 		}
 
@@ -753,7 +754,7 @@ static char *_fetch_string(char **src, const int delimiter)
 
 /* Parse a device message from the daemon. */
 static int _parse_message(struct dm_event_daemon_message *msg, char **dso_name,
-			 char **uuid, enum dm_event_mask *evmask)
+			 char **uuid, unsigned *evmask)
 {
 	char *id;
 	char *p = msg->data;
@@ -778,7 +779,7 @@ int dm_event_get_registered_device(struct dm_event_handler *dmevh, int next)
 	int ret = 0;
 	const char *uuid = NULL;
 	char *reply_dso = NULL, *reply_uuid = NULL;
-	enum dm_event_mask reply_mask = 0;
+	unsigned reply_mask = 0;
 	struct dm_task *dmt = NULL;
 	struct dm_event_daemon_message msg = { 0 };
 	struct dm_info info;
