@@ -264,19 +264,19 @@ static void format_info_r_action(char *line, char *r_name, char *r_type)
 
 static void format_info_line(char *line, char *r_name, char *r_type)
 {
-	if (!strncmp(line, "info=structs ", strlen("info=structs "))) {
+	if (!strncmp(line, "info=structs ", sizeof("info=structs ") - 1)) {
 		/* only print this in the raw info dump */
 
-	} else if (!strncmp(line, "info=client ", strlen("info=client "))) {
+	} else if (!strncmp(line, "info=client ", sizeof("info=client ") - 1)) {
 		save_client_info(line);
 
-	} else if (!strncmp(line, "info=ls ", strlen("info=ls "))) {
+	} else if (!strncmp(line, "info=ls ", sizeof("info=ls ") - 1)) {
 		format_info_ls(line);
 
-	} else if (!strncmp(line, "info=ls_action ", strlen("info=ls_action "))) {
+	} else if (!strncmp(line, "info=ls_action ", sizeof("info=ls_action ") - 1)) {
 		format_info_ls_action(line);
 
-	} else if (!strncmp(line, "info=r ", strlen("info=r "))) {
+	} else if (!strncmp(line, "info=r ", sizeof("info=r ") - 1)) {
 		/*
 		 * r_name/r_type are reset when a new resource is found.
 		 * They are reused for the lock and action lines that
@@ -286,11 +286,11 @@ static void format_info_line(char *line, char *r_name, char *r_type)
 		memset(r_type, 0, MAX_NAME+1);
 		format_info_r(line, r_name, r_type);
 
-	} else if (!strncmp(line, "info=lk ", strlen("info=lk "))) {
+	} else if (!strncmp(line, "info=lk ", sizeof("info=lk ") - 1)) {
 		/* will use info from previous r */
 		format_info_lk(line, r_name, r_type);
 
-	} else if (!strncmp(line, "info=r_action ", strlen("info=r_action "))) {
+	} else if (!strncmp(line, "info=r_action ", sizeof("info=r_action ") - 1)) {
 		/* will use info from previous r */
 		format_info_r_action(line, r_name, r_type);
 	} else {
