@@ -52,7 +52,7 @@ static pthread_key_t key;
 
 static const char *_strerror_r(int errnum, struct lvmpolld_thread_data *data)
 {
-#ifdef _GNU_SOURCE
+#if defined(_GNU_SOURCE) && defined(STRERROR_R_CHAR_P)
 	return strerror_r(errnum, data->buf, sizeof(data->buf)); /* never returns NULL */
 #elif (_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600)
 	return strerror_r(errnum, data->buf, sizeof(data->buf)) ? "" : data->buf;
@@ -149,7 +149,7 @@ static void _lvmpolld_global_unlock(struct lvmpolld_state *ls)
 static int _fini(struct daemon_state *s)
 {
 	int done;
-	const struct timespec t = { .tv_nsec = 250000000 }; /* .25 sec */
+	const struct timespec t = { .tv_nsec = 10000000 }; /* .01 sec */
 	struct lvmpolld_state *ls = s->private;
 
 	DEBUGLOG(s, "fini");
@@ -236,9 +236,7 @@ static int poll_for_output(struct lvmpolld_lv *pdlv, struct lvmpolld_thread_data
 	}
 
 	while (1) {
-		do {
-			r = poll(fds, 2, pdlv_get_timeout(pdlv) * 1000);
-		} while (r < 0 && errno == EINTR);
+		r = poll(fds, 2, pdlv_get_timeout(pdlv) * 1000);
 
 		DEBUGLOG(pdlv->ls, "%s: %s %d", PD_LOG_PREFIX, "poll() returned", r);
 		if (r < 0) {
@@ -374,7 +372,7 @@ static void debug_print(struct lvmpolld_state *ls, const char * const* ptr)
 
 static void *fork_and_poll(void *args)
 {
-	int outfd, errfd, state;
+	int outfd, errfd, state = 0;
 	struct lvmpolld_thread_data *data;
 	pid_t r;
 
@@ -555,14 +553,15 @@ static struct lvmpolld_lv *construct_pdlv(request req, struct lvmpolld_state *ls
 				     const char *interval, const char *id,
 				     const char *vgname, const char *lvname,
 				     const char *sysdir, enum poll_type type,
-				     unsigned abort_polling, unsigned uinterval)
+				     unsigned abort_polling, unsigned uinterval,
+				     const char *devicesfile)
 {
 	const char **cmdargv, **cmdenvp;
 	struct lvmpolld_lv *pdlv;
 	unsigned handle_missing_pvs = daemon_request_int(req, LVMPD_PARM_HANDLE_MISSING_PVS, 0);
 
 	pdlv = pdlv_create(ls, id, vgname, lvname, sysdir, type,
-			   interval, uinterval, pdst);
+			   interval, uinterval, pdst, devicesfile);
 
 	if (!pdlv) {
 		ERROR(ls, "%s: %s", PD_LOG_PREFIX, "failed to create internal LV data structure.");
@@ -621,6 +620,7 @@ static response poll_init(client_handle h, struct lvmpolld_state *ls, request re
 	const char *lvname = daemon_request_str(req, LVMPD_PARM_LVNAME, NULL);
 	const char *vgname = daemon_request_str(req, LVMPD_PARM_VGNAME, NULL);
 	const char *sysdir = daemon_request_str(req, LVMPD_PARM_SYSDIR, NULL);
+	const char *devicesfile = daemon_request_str(req, LVMPD_PARM_DEVICESFILE, NULL);
 	unsigned abort_polling = daemon_request_int(req, LVMPD_PARM_ABORT, 0);
 
 	assert(type < POLL_TYPE_MAX);
@@ -680,7 +680,7 @@ static response poll_init(client_handle h, struct lvmpolld_state *ls, request re
 		pdlv->init_rq_count++; /* safe. protected by store lock */
 	} else {
 		pdlv = construct_pdlv(req, ls, pdst, interval, id, vgname,
-				      lvname, sysdir, type, abort_polling, 2 * uinterval);
+				      lvname, sysdir, type, abort_polling, 2 * uinterval, devicesfile);
 		if (!pdlv) {
 			pdst_unlock(pdst);
 			free(id);
@@ -781,7 +781,7 @@ struct log_line_baton {
 	const char *prefix;
 };
 
-daemon_handle _lvmpolld = { .error = 0 };
+static daemon_handle _lvmpolld = { .error = 0 };
 
 static daemon_handle _lvmpolld_open(const char *socket)
 {
@@ -867,14 +867,14 @@ enum action_index {
 	ACTION_MAX /* keep at the end */
 };
 
-static const action_fn_t actions[ACTION_MAX] = { [ACTION_DUMP] = action_dump };
-
 static int _make_action(enum action_index idx, void *args)
 {
-	return idx < ACTION_MAX ? actions[idx](args) : 0;
+	static const action_fn_t _actions[ACTION_MAX] = { [ACTION_DUMP] = action_dump };
+
+	return idx < ACTION_MAX ? _actions[idx](args) : 0;
 }
 
-static int _lvmpolld_client(const char *socket, unsigned action)
+static int _lvmpolld_client(const char *socket, enum action_index action)
 {
 	int r;
 
@@ -892,10 +892,9 @@ static int _lvmpolld_client(const char *socket, unsigned action)
 	return r ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
-static int action_idx = ACTION_MAX;
-static struct option long_options[] = {
+static const struct option _long_options[] = {
 	/* Have actions always at the beginning of the array. */
-	{"dump",	no_argument,		&action_idx,	ACTION_DUMP }, /* or an option_index ? */
+	{"dump",	no_argument,		0,		ACTION_DUMP }, /* or an option_index ? */
 
 	/* other options */
 	{"binary",	required_argument,	0,		'B' },
@@ -914,8 +913,8 @@ int main(int argc, char *argv[])
 	int opt;
 	int option_index = 0;
 	int client = 0, server = 0;
-	unsigned action = ACTION_MAX;
-	struct timeval timeout;
+	enum action_index action = ACTION_MAX;
+	struct timespec timeout;
 	daemon_idle di = { .ptimeout = &timeout };
 	struct lvmpolld_state ls = { .log_config = "" };
 	daemon_state s = {
@@ -930,16 +929,16 @@ int main(int argc, char *argv[])
 		.socket_path = getenv("LVM_LVMPOLLD_SOCKET") ?: LVMPOLLD_SOCKET,
 	};
 
-	while ((opt = getopt_long(argc, argv, "fhVl:p:s:B:t:", long_options, &option_index)) != -1) {
+	while ((opt = getopt_long(argc, argv, "fhVl:p:s:B:t:", _long_options, &option_index)) != -1) {
 		switch (opt) {
 		case 0 :
-			if (action < ACTION_MAX) {
+			if (action != ACTION_MAX) {
 				fprintf(stderr, "Can't perform more actions. Action already requested: %s\n",
-					long_options[action].name);
+					_long_options[action].name);
 				_usage(argv[0], stderr);
 				exit(EXIT_FAILURE);
 			}
-			action = action_idx;
+			action = ACTION_DUMP;
 			client = 1;
 			break;
 		case '?':
