@@ -175,12 +175,11 @@ struct dm_names {
 
 struct dm_active_device {
 	struct dm_list list;
-	int major;
-	int minor;
-	char *name;	/* device name */
+	dev_t devno;
+	const char *name;	/* device name */
 
 	uint32_t event_nr; /* valid when DM_DEVICE_LIST_HAS_EVENT_NR is set */
-	char *uuid;	/* valid uuid when DM_DEVICE_LIST_HAS_UUID is set */
+	const char *uuid;	/* valid uuid when DM_DEVICE_LIST_HAS_UUID is set */
 };
 
 struct dm_versions {
@@ -230,13 +229,6 @@ struct dm_names *dm_task_get_names(struct dm_task *dmt);
 #define DM_DEVICE_LIST_HAS_UUID		2
 int dm_task_get_device_list(struct dm_task *dmt, struct dm_list **devs_list,
 			    unsigned *devs_features);
-/*
- * -1: no idea about uuid (not provided by DM_DEVICE_LIST ioctl)
- *  0: uuid not present
- *  1: listed and dm_active_device will be set for not NULL pointer
- */
-int dm_device_list_find_by_uuid(struct dm_list *devs_list, const char *uuid,
-				const struct dm_active_device **dev);
 /* Release all associated memory with list of active DM devices */
 void dm_device_list_destroy(struct dm_list **devs_list);
 
