@@ -198,6 +198,7 @@ static int _get_proc_number(const char *file, const char *name,
 	char *line = NULL;
 	size_t len;
 	uint32_t num;
+	unsigned blocksection = (strcmp(file, PROC_DEVICES) == 0) ? 0 : 1;
 
 	if (!(fl = fopen(file, "r"))) {
 		log_sys_error("fopen", file);
@@ -205,7 +206,9 @@ static int _get_proc_number(const char *file, const char *name,
 	}
 
 	while (getline(&line, &len, fl) != -1) {
-		if (sscanf(line, "%u %255s\n", &num, &nm[0]) == 2) {
+		if (!blocksection && (line[0] == 'B'))
+			blocksection = 1;
+		else if (sscanf(line, "%u %255s\n", &num, &nm[0]) == 2) {
 			if (!strcmp(name, nm)) {
 				if (number) {
 					*number = num;
@@ -779,19 +782,12 @@ static int _check_has_event_nr(void) {
 	return _has_event_nr;
 }
 
-struct dm_device_list {
-	struct dm_list list;
-	unsigned count;
-	unsigned features;
-	struct dm_hash_table *uuids;
-};
-
 int dm_task_get_device_list(struct dm_task *dmt, struct dm_list **devs_list,
 			    unsigned *devs_features)
 {
 	struct dm_names *names, *names1;
 	struct dm_active_device *dm_dev, *dm_new_dev;
-	struct dm_device_list *devs;
+	struct dm_list *devs;
 	unsigned next = 0;
 	uint32_t *event_nr;
 	char *uuid_ptr;
@@ -812,12 +808,12 @@ int dm_task_get_device_list(struct dm_task *dmt, struct dm_list **devs_list,
 		} while (next);
 	}
 
-	if (!(devs = malloc(sizeof(*devs) + (cnt ? cnt * sizeof(*dm_dev) + (char*)names1 - (char*)names + 256 : 0))))
+	/* buffer for devs +  sorted ptrs + dm_devs + aligned strings */
+	if (!(devs = malloc(sizeof(*devs) + cnt * (2 * sizeof(void*) + sizeof(*dm_dev)) +
+			    (cnt ? (char*)names1 - (char*)names + 256 : 0))))
 		return_0;
 
-	dm_list_init(&devs->list);
-	devs->count = cnt;
-	devs->uuids = NULL;
+	dm_list_init(devs);
 
 	if (!cnt) {
 		/* nothing in the list -> mark all features present */
@@ -825,27 +821,22 @@ int dm_task_get_device_list(struct dm_task *dmt, struct dm_list **devs_list,
 		goto out; /* nothing else to do */
 	}
 
-	dm_dev = (struct dm_active_device *) (devs + 1);
+	/* Shift position where to store individual dm_devs */
+	dm_dev = (struct dm_active_device *) ((long*) (devs + 1) + cnt);
 
 	do {
 		names = (struct dm_names *)((char *) names + next);
 
-		dm_dev->major = MAJOR(names->dev);
-		dm_dev->minor = MINOR(names->dev);
-		dm_dev->name = (char*)(dm_dev + 1);
+		dm_dev->devno = (dev_t) names->dev;
+		dm_dev->name = (const char *)(dm_dev + 1);
 		dm_dev->event_nr = 0;
-		dm_dev->uuid = NULL;
+		dm_dev->uuid = "";
 
 		len = strlen(names->name) + 1;
-		memcpy(dm_dev->name, names->name, len);
+		memcpy((char*)dm_dev->name, names->name, len);
 
 		dm_new_dev = _align_ptr((char*)(dm_dev + 1) + len);
 		if (_check_has_event_nr()) {
-			/* Hash for UUIDs with some more bits to reduce colision count */
-			if (!devs->uuids && !(devs->uuids = dm_hash_create(cnt * 8))) {
-				free(devs);
-				return_0;
-			}
 
 			*devs_features |= DM_DEVICE_LIST_HAS_EVENT_NR;
 			event_nr = _align_ptr(names->name + len);
@@ -854,55 +845,29 @@ int dm_task_get_device_list(struct dm_task *dmt, struct dm_list **devs_list,
 			if ((event_nr[1] & DM_NAME_LIST_FLAG_HAS_UUID)) {
 				*devs_features |= DM_DEVICE_LIST_HAS_UUID;
 				uuid_ptr = _align_ptr(event_nr + 2);
-				dm_dev->uuid = (char*) dm_new_dev;
 				len = strlen(uuid_ptr) + 1;
+				memcpy(dm_new_dev, uuid_ptr, len);
+				dm_dev->uuid = (const char *) dm_new_dev;
 				dm_new_dev = _align_ptr((char*)dm_new_dev + len);
-				memcpy(dm_dev->uuid, uuid_ptr, len);
-				if (!dm_hash_insert(devs->uuids, dm_dev->uuid, dm_dev))
-					return_0; // FIXME
-#if 0
-				log_debug("Active %s (%s) %d:%d event:%u",
-					  dm_dev->name, dm_dev->uuid,
-					  dm_dev->major, dm_dev->minor, dm_dev->event_nr);
-#endif
 			}
 		}
 
-		dm_list_add(&devs->list, &dm_dev->list);
+		dm_list_add(devs, &dm_dev->list);
 		dm_dev = dm_new_dev;
 		next = names->next;
 	} while (next);
 
     out:
-	*devs_list = (struct dm_list *)devs;
+	*devs_list = devs;
 
 	return 1;
 }
 
-int dm_device_list_find_by_uuid(struct dm_list *devs_list, const char *uuid,
-				const struct dm_active_device **dev)
-{
-	struct dm_device_list *devs = (struct dm_device_list *) devs_list;
-	struct dm_active_device *dm_dev;
-
-	if (devs->uuids &&
-	    (dm_dev = dm_hash_lookup(devs->uuids, uuid))) {
-		if (dev)
-			*dev = dm_dev;
-		return 1;
-	}
-
-	return 0;
-}
-
 void dm_device_list_destroy(struct dm_list **devs_list)
 {
 	struct dm_device_list *devs = (struct dm_device_list *) *devs_list;
 
 	if (devs) {
-		if (devs->uuids)
-			dm_hash_destroy(devs->uuids);
-
 		free(devs);
 		*devs_list = NULL;
 	}
