@@ -418,7 +418,7 @@ int add_areas_line(struct dev_manager *dm, struct lv_segment *seg,
 {
         return 0;
 }
-int device_is_usable(struct cmd_context *cmd, struct device *dev, struct dev_usable_check_params check, int *is_lv)
+int dm_device_is_usable(struct cmd_context *cmd, struct device *dev, struct dev_usable_check_params check, int *is_lv)
 {
         return 0;
 }
@@ -672,13 +672,13 @@ int target_present(struct cmd_context *cmd, const char *target_name,
 				      &maj, &min, &patchlevel);
 }
 
-int get_device_list(const struct volume_group *vg, struct dm_list **devs,
-		    unsigned *devs_features)
+int get_dm_active_devices(const struct volume_group *vg, struct dm_list **devs,
+			  unsigned *devs_features)
 {
 	if (!activation())
 		return 0;
 
-	return dev_manager_get_device_list(NULL, devs, devs_features);
+	return dev_manager_get_dm_active_devices(NULL, devs, devs_features);
 }
 
 /*
@@ -2461,7 +2461,6 @@ int lv_deactivate(struct cmd_context *cmd, const char *lvid_s, const struct logi
 	const struct lv_activate_opts laopts = { .skip_in_use = 1 };
 	struct dm_list *snh;
 	int r = 0;
-	unsigned tmp_state;
 
 	if (!activation())
 		return 1;
@@ -2534,9 +2533,6 @@ int lv_deactivate(struct cmd_context *cmd, const char *lvid_s, const struct logi
 	}
 	critical_section_dec(cmd, "deactivated");
 
-	tmp_state = cmd->disable_dm_devs;
-	cmd->disable_dm_devs = 1;
-
 	if (!lv_info(cmd, lv, 0, &info, 0, 0) || info.exists) {
 		/* Turn into log_error, but we do not log error */
 		log_debug_activation("Deactivated volume is still %s present.",
@@ -2544,7 +2540,6 @@ int lv_deactivate(struct cmd_context *cmd, const char *lvid_s, const struct logi
 		r = 0;
 	}
 
-	cmd->disable_dm_devs = tmp_state;
 out:
 
 	return r;
