@@ -211,12 +211,12 @@ struct cmd_context {
 	unsigned event_activation:1;		/* whether event_activation is set */
 	unsigned udevoutput:1;
 	unsigned online_vg_file_removed:1;
-	unsigned disable_dm_devs:1;		/* temporarily disable use of dm devs cache */
 	unsigned filter_regex_set_preferred_name_disable:1; /* prevent dev_set_preferred_name */
 	unsigned device_ids_check_product_uuid:1;
 	unsigned device_ids_check_hostname:1;
 	unsigned device_ids_refresh_trigger:1;
 	unsigned device_ids_invalid:1;
+	unsigned device_ids_auto_import:1;
 	unsigned get_vgname_from_options:1;     /* used by lvconvert */
 
 	/*
@@ -230,8 +230,6 @@ struct cmd_context {
 	const char *devicesfile;                /* from --devicesfile option */
 	struct dm_list deviceslist;             /* from --devices option, struct dm_str_list */
 
-	struct dm_list *cache_dm_devs;		/* cache with UUIDs from DM_DEVICE_LIST (when available) */
-
 	/*
 	 * Configuration.
 	 */
@@ -286,6 +284,7 @@ struct cmd_context {
 	struct dm_list pending_delete;		/* list of LVs for removal */
 	struct dm_pool *pending_delete_mem;	/* memory pool for pending deletes */
 	struct vdo_convert_params *lvcreate_vcp;/* params for LV to VDO conversion */
+	uint32_t lockopt;			/* LOCKOPT_* from --lockopt string */
 };
 
 /*
