@@ -49,7 +49,6 @@ struct config_file {
 	off_t st_size;
 	char *filename;
 	int exists;
-	int keep_open;
 	struct device *dev;
 };
 
@@ -104,7 +103,7 @@ static inline int _is_file_based_config_source(config_source_t source)
  */
 struct dm_config_tree *config_open(config_source_t source,
 				   const char *filename,
-				   int keep_open)
+				   int unused)
 {
 	struct dm_config_tree *cft = dm_config_create();
 	struct config_source *cs;
@@ -124,7 +123,6 @@ struct dm_config_tree *config_open(config_source_t source,
 			goto fail;
 		}
 
-		cf->keep_open = keep_open;
 		if (filename &&
 		    !(cf->filename = dm_pool_strdup(cft->mem, filename))) {
 			log_error("Failed to duplicate filename.");
@@ -241,22 +239,8 @@ int config_file_changed(struct dm_config_tree *cft)
 
 void config_destroy(struct dm_config_tree *cft)
 {
-	struct config_source *cs;
-	struct config_file *cf;
-
-	if (!cft)
-		return;
-
-	cs = dm_config_get_custom(cft);
-
-	if (_is_file_based_config_source(cs->type)) {
-		cf = cs->source.file;
-		if (cf && cf->dev)
-			if (!dev_close(cf->dev))
-				stack;
-	}
-
-	dm_config_destroy(cft);
+	if (cft)
+		dm_config_destroy(cft);
 }
 
 struct dm_config_tree *config_file_open_and_read(const char *config_file,
@@ -281,7 +265,7 @@ struct dm_config_tree *config_file_open_and_read(const char *config_file,
 	}
 
 	log_very_verbose("Loading config file: %s", config_file);
-	if (!config_file_read(cft)) {
+	if (!config_file_read_from_file(cft)) {
 		log_error("Failed to load config file %s", config_file);
 		goto bad;
 	}
@@ -605,12 +589,15 @@ int config_file_read_fd(struct dm_config_tree *cft, struct device *dev, dev_io_r
 	return r;
 }
 
-int config_file_read(struct dm_config_tree *cft)
+int config_file_read_from_file(struct dm_config_tree *cft)
 {
 	const char *filename = NULL;
 	struct config_source *cs = dm_config_get_custom(cft);
 	struct config_file *cf;
 	struct stat info;
+	struct device fake_dev = { 0 };
+	struct dm_str_list *alias;
+	int fd;
 	int r;
 
 	if (!config_file_check(cft, &filename, &info))
@@ -622,25 +609,39 @@ int config_file_read(struct dm_config_tree *cft)
 
 	cf = cs->source.file;
 
-	if (!cf->dev) {
-		if (!(cf->dev = dev_create_file(filename, NULL, NULL, 1)))
-			return_0;
+	if (cf->dev)
+		return_0;
 
-		if (!dev_open_readonly_buffered(cf->dev)) {
-			dev_destroy_file(cf->dev);
-			cf->dev = NULL;
-			return_0;
-		}
+	/* fixme: get rid of fake dev and just
+	   add separate code paths for files */
+
+	if (!(alias = zalloc(sizeof(*alias))))
+		return_0;
+	if (!(alias->str = strdup(filename))) {
+		free(alias);
+		return_0;
 	}
+	dev_init(&fake_dev);
+	fake_dev.flags = DEV_REGULAR;
+	dm_list_add(&fake_dev.aliases, &alias->list);
+
+	if ((fd = open(filename, O_RDONLY, 0777)) < 0) {
+		log_error("Failed to open config file %s.", filename);
+		free((void*)alias->str);
+		free((void*)alias);
+		return_0;
+	}
+	fake_dev.fd = fd;
+	cf->dev = &fake_dev;
 
 	r = config_file_read_fd(cft, cf->dev, DEV_IO_MDA_CONTENT, 0, (size_t) info.st_size, 0, 0,
 				(checksum_fn_t) NULL, 0, 0, 0);
 
-	if (!cf->keep_open) {
-		if (!dev_close(cf->dev))
-			stack;
-		cf->dev = NULL;
-	}
+	free((void*)alias->str);
+	free((void*)alias);
+	close(fd);
+
+	cf->dev = NULL;
 
 	return r;
 }
@@ -687,7 +688,7 @@ int config_def_get_path(char *buf, size_t buf_size, int id)
 	return _cfg_def_make_path(buf, buf_size, id, cfg_def_get_item_p(id), 0);
 }
 
-static void _get_type_name(char *buf, size_t buf_size, cfg_def_type_t type)
+static void _get_type_name(char *buf, size_t buf_size, unsigned type)
 {
 	(void) dm_snprintf(buf, buf_size, "%s%s%s%s%s%s",
 			   (type & CFG_TYPE_ARRAY) ?
@@ -700,8 +701,8 @@ static void _get_type_name(char *buf, size_t buf_size, cfg_def_type_t type)
 			   (type & CFG_TYPE_STRING) ? " string" : "");
 }
 
-static void _log_type_error(const char *path, cfg_def_type_t actual,
-			    cfg_def_type_t expected, int suppress_messages)
+static void _log_type_error(const char *path, unsigned actual, unsigned expected,
+			    int suppress_messages)
 {
 	static char actual_type_name[128];
 	static char expected_type_name[128];
@@ -941,7 +942,7 @@ static int _check_value_differs_from_default(struct cft_check_handle *handle,
 				diff = (v_def && (v_def->type != DM_CFG_EMPTY_ARRAY));
 				break;
 			default:
-				log_error(INTERNAL_ERROR "inconsistent state reached in _check_value_differs_from_default");
+				log_error(INTERNAL_ERROR "Inconsistent state reached in _check_value_differs_from_default.");
 				return 0;
 		}
 	}
@@ -2333,7 +2334,7 @@ int load_profile(struct cmd_context *cmd, struct profile *profile) {
 	static char profile_path[PATH_MAX];
 
 	if (critical_section()) {
-		log_error(INTERNAL_ERROR "trying to load profile %s "
+		log_error(INTERNAL_ERROR "Trying to load profile %s "
 			  "in critical section.", profile->name);
 		return 0;
 	}
