@@ -22,6 +22,7 @@
 #include "lib/device/bcache.h"
 #include "lib/label/label.h"
 #include "lib/commands/toolcontext.h"
+#include "lib/activate/activate.h"
 #include "device_mapper/misc/dm-ioctl.h"
 
 #ifdef BLKID_WIPING_SUPPORT
@@ -50,33 +51,15 @@ int dev_is_nvme(struct dev_types *dt, struct device *dev)
 	return (dev->flags & DEV_IS_NVME) ? 1 : 0;
 }
 
-int dev_is_lv(struct device *dev)
+int dev_is_lv(struct cmd_context *cmd, struct device *dev)
 {
-	FILE *fp;
-	char path[PATH_MAX];
-	char buffer[64];
-	int ret = 0;
+	char buffer[128];
 
-	if (dm_snprintf(path, sizeof(path), "%sdev/block/%d:%d/dm/uuid",
-			dm_sysfs_dir(),
-			(int) MAJOR(dev->dev),
-			(int) MINOR(dev->dev)) < 0) {
-		log_warn("Sysfs dm uuid path for %s is too long.", dev_name(dev));
-		return 0;
-	}
+	if (dev_dm_uuid(cmd, dev, buffer, sizeof(buffer)) &&
+	    !strncmp(buffer, UUID_PREFIX, sizeof(UUID_PREFIX) - 1))
+                return 1;
 
-	if (!(fp = fopen(path, "r")))
-		return 0;
-
-	if (!fgets(buffer, sizeof(buffer), fp))
-		log_debug("Failed to read %s.", path);
-	else if (!strncmp(buffer, "LVM-", 4))
-		ret = 1;
-
-	if (fclose(fp))
-		log_sys_debug("fclose", path);
-
-	return ret;
+	return 0;
 }
 
 int dev_is_used_by_active_lv(struct cmd_context *cmd, struct device *dev, int *used_by_lv_count,
@@ -89,9 +72,9 @@ int dev_is_used_by_active_lv(struct cmd_context *cmd, struct device *dev, int *u
 	DIR *d;
 	struct dirent *dirent;
 	char *holder_name;
-	int dm_dev_major, dm_dev_minor;
-	size_t lvm_prefix_len = sizeof(UUID_PREFIX) - 1;
-	size_t lvm_uuid_len = sizeof(UUID_PREFIX) - 1 + 2 * ID_LEN;
+	unsigned dm_dev_major, dm_dev_minor;
+	const size_t lvm_prefix_len = sizeof(UUID_PREFIX) - 1;
+	const size_t lvm_uuid_len = lvm_prefix_len + 2 * ID_LEN;
 	size_t uuid_len;
 	int used_count = 0;
 	char *used_name = NULL;
@@ -103,7 +86,8 @@ int dev_is_used_by_active_lv(struct cmd_context *cmd, struct device *dev, int *u
 	 * sysfs "holders" dir.
 	 */
 
-	if (dm_snprintf(holders_path, sizeof(holders_path), "%sdev/block/%d:%d/holders/", dm_sysfs_dir(), (int) MAJOR(dev->dev), (int) MINOR(dev->dev)) < 0) {
+	if (dm_snprintf(holders_path, sizeof(holders_path), "%sdev/block/%u:%u/holders/",
+			dm_sysfs_dir(), MAJOR(dev->dev), MINOR(dev->dev)) < 0) {
 		log_error("%s: dm_snprintf failed for path to holders directory.", dev_name(dev));
 		return 0;
 	}
@@ -132,25 +116,21 @@ int dev_is_used_by_active_lv(struct cmd_context *cmd, struct device *dev, int *u
 		if (stat(dm_dev_path, &info))
 			continue;
 
-		dm_dev_major = (int)MAJOR(info.st_rdev);
-		dm_dev_minor = (int)MINOR(info.st_rdev);
+		dm_dev_major = MAJOR(info.st_rdev);
+		dm_dev_minor = MINOR(info.st_rdev);
         
 		if (dm_dev_major != cmd->dev_types->device_mapper_major)
 			continue;
 
 		/*
 		 * if "dm-1" is a dm device, then check if it's an LVM LV
-		 * by reading /sys/block/<holder_name>/dm/uuid and seeing
-		 * if the uuid begins with LVM-
-		 * UUID_PREFIX is "LVM-"
+		 * by reading DM status and seeing if the uuid begins
+		 * with UUID_PREFIX  ("LVM-")
 		 */
-
-		dm_uuid[0] = '\0';
-
-		if (!get_dm_uuid_from_sysfs(dm_uuid, sizeof(dm_uuid), dm_dev_major, dm_dev_minor))
+		if (!devno_dm_uuid(cmd, dm_dev_major, dm_dev_minor, dm_uuid, sizeof(dm_uuid)))
 			continue;
 
-		if (!strncmp(dm_uuid, UUID_PREFIX, 4))
+		if (!strncmp(dm_uuid, UUID_PREFIX, lvm_prefix_len))
 			used_count++;
 
 		if (used_by_dm_name && !used_name)
@@ -270,45 +250,46 @@ struct dev_types *create_dev_types(const char *proc_dir,
 		while (line[i] == ' ')
 			i++;
 
-		/* Look for md device */
-		if (!strncmp("md", line + i, 2) && isspace(*(line + i + 2)))
-			dt->md_major = line_maj;
-
-		/* Look for blkext device */
-		if (!strncmp("blkext", line + i, 6) && isspace(*(line + i + 6)))
-			dt->blkext_major = line_maj;
-
-		/* Look for drbd device */
-		if (!strncmp("drbd", line + i, 4) && isspace(*(line + i + 4)))
-			dt->drbd_major = line_maj;
-
-		/* Look for DASD */
-		if (!strncmp("dasd", line + i, 4) && isspace(*(line + i + 4)))
-			dt->dasd_major = line_maj;
-
-		/* Look for EMC powerpath */
-		if (!strncmp("emcpower", line + i, 8) && isspace(*(line + i + 8)))
-			dt->emcpower_major = line_maj;
-
-		/* Look for Veritas Dynamic Multipathing */
-		if (!strncmp("VxDMP", line + i, 5) && isspace(*(line + i + 5)))
-			dt->vxdmp_major = line_maj;
-
-		if (!strncmp("loop", line + i, 4) && isspace(*(line + i + 4)))
-			dt->loop_major = line_maj;
-
-		if (!strncmp("power2", line + i, 6) && isspace(*(line + i + 6)))
-			dt->power2_major = line_maj;
-
-		/* Look for device-mapper device */
-		/* FIXME Cope with multiple majors */
-		if (!strncmp("device-mapper", line + i, 13) && isspace(*(line + i + 13)))
-			dt->device_mapper_major = line_maj;
-
 		/* Major is SCSI device */
 		if (!strncmp("sd", line + i, 2) && isspace(*(line + i + 2)))
 			dt->dev_type_array[line_maj].flags |= PARTITION_SCSI_DEVICE;
 
+		else if (!strncmp("loop", line + i, 4) && isspace(*(line + i + 4)))
+			dt->loop_major = line_maj;
+
+		/* Look for device-mapper device */
+		/* FIXME Cope with multiple majors */
+		else if (!strncmp("device-mapper", line + i, 13) && isspace(*(line + i + 13)))
+			dt->device_mapper_major = line_maj;
+
+		/* Look for md device */
+		else if (!strncmp("md", line + i, 2) && isspace(*(line + i + 2)))
+			dt->md_major = line_maj;
+
+		/* Look for blkext device */
+		else if (!strncmp("blkext", line + i, 6) && isspace(*(line + i + 6)))
+			dt->blkext_major = line_maj;
+
+		/* Look for drbd device */
+		else if (!strncmp("drbd", line + i, 4) && isspace(*(line + i + 4)))
+			dt->drbd_major = line_maj;
+
+		/* Look for DASD */
+		else if (!strncmp("dasd", line + i, 4) && isspace(*(line + i + 4)))
+			dt->dasd_major = line_maj;
+
+		/* Look for EMC powerpath */
+		else if (!strncmp("emcpower", line + i, 8) && isspace(*(line + i + 8)))
+			dt->emcpower_major = line_maj;
+
+		/* Look for Veritas Dynamic Multipathing */
+		else if (!strncmp("VxDMP", line + i, 5) && isspace(*(line + i + 5)))
+			dt->vxdmp_major = line_maj;
+
+		else if (!strncmp("power2", line + i, 6) && isspace(*(line + i + 6)))
+			dt->power2_major = line_maj;
+
+
 		/* Go through the valid device names and if there is a
 		   match store max number of partitions */
 		for (j = 0; _dev_known_types[j].name[0]; j++) {
@@ -363,7 +344,7 @@ struct dev_types *create_dev_types(const char *proc_dir,
 	}
 
 	if (fclose(pd))
-		log_sys_error("fclose", proc_devices);
+		log_sys_debug("fclose", proc_devices);
 
 	return dt;
 bad:
@@ -459,10 +440,8 @@ static int _loop_is_with_partscan(struct device *dev)
 	char path[PATH_MAX];
 	char buffer[64];
 
-	if (dm_snprintf(path, sizeof(path), "%sdev/block/%d:%d/loop/partscan",
-			dm_sysfs_dir(),
-			(int) MAJOR(dev->dev),
-			(int) MINOR(dev->dev)) < 0) {
+	if (dm_snprintf(path, sizeof(path), "%sdev/block/%u:%u/loop/partscan",
+			dm_sysfs_dir(), MAJOR(dev->dev), MINOR(dev->dev)) < 0) {
 		log_warn("Sysfs path for partscan is too long.");
 		return 0;
 	}
@@ -495,8 +474,8 @@ int dev_get_partition_number(struct device *dev, int *num)
 		return 1;
 	}
 
-	if (dm_snprintf(path, sizeof(path), "%sdev/block/%d:%d/partition",
-			dm_sysfs_dir(), (int)MAJOR(devt), (int)MINOR(devt)) < 0) {
+	if (dm_snprintf(path, sizeof(path), "%sdev/block/%u:%u/partition",
+			dm_sysfs_dir(), MAJOR(devt), MINOR(devt)) < 0) {
 		log_error("Failed to create sysfs path for %s", dev_name(dev));
 		return 0;
 	}
@@ -544,11 +523,11 @@ static int _has_sys_partition(struct device *dev)
 {
 	char path[PATH_MAX];
 	struct stat info;
-	int major = (int) MAJOR(dev->dev);
-	int minor = (int) MINOR(dev->dev);
+	unsigned major = MAJOR(dev->dev);
+	unsigned minor = MINOR(dev->dev);
 
 	/* check if dev is a partition */
-	if (dm_snprintf(path, sizeof(path), "%sdev/block/%d:%d/partition",
+	if (dm_snprintf(path, sizeof(path), "%sdev/block/%u:%u/partition",
 			dm_sysfs_dir(), major, minor) < 0) {
 		log_warn("WARNING: %s: partition path is too long.", dev_name(dev));
 		return 0;
@@ -724,8 +703,8 @@ int dev_is_partitioned(struct cmd_context *cmd, struct device *dev)
  */
 int dev_get_primary_dev(struct dev_types *dt, struct device *dev, dev_t *result)
 {
-	int major = (int) MAJOR(dev->dev);
-	int minor = (int) MINOR(dev->dev);
+	unsigned major = MAJOR(dev->dev);
+	unsigned minor = MINOR(dev->dev);
 	char path[PATH_MAX];
 	char temp_path[PATH_MAX];
 	char buffer[64];
@@ -775,7 +754,7 @@ int dev_get_primary_dev(struct dev_types *dt, struct device *dev, dev_t *result)
 	 * - basename ../../block/md0/md0  = md0
 	 * Parent's 'dev' sysfs attribute  = /sys/block/md0/dev
 	 */
-	if (dm_snprintf(path, sizeof(path), "%sdev/block/%d:%d",
+	if (dm_snprintf(path, sizeof(path), "%sdev/block/%u:%u",
 			dm_sysfs_dir(), major, minor) < 0) {
 		log_warn("WARNING: %s: major:minor sysfs path is too long.", dev_name(dev));
 		return 0;
@@ -827,7 +806,7 @@ int fs_block_size_and_type(const char *pathname, uint32_t *fs_block_size_bytes,
 {
 	blkid_probe probe = NULL;
 	const char *type_str = NULL, *size_str = NULL;
-	size_t len;
+	size_t len = 0;
 	int ret = 1;
 	int rc;
 
@@ -887,8 +866,8 @@ out:
 int fs_get_blkid(const char *pathname, struct fs_info *fsi)
 {
 	blkid_probe probe = NULL;
-	const char *str;
-	size_t len;
+	const char *str = "";
+	size_t len = 0;
 	uint64_t fslastblock = 0;
 	unsigned int fsblocksize = 0;
 	int rc;
@@ -915,7 +894,7 @@ int fs_get_blkid(const char *pathname, struct fs_info *fsi)
 		return 0;
 	} else if (rc == 1) {
 		/* no file system on the device */
-		log_print("No file system found on %s.", pathname);
+		log_print_unless_silent("No file system found on %s.", pathname);
 		fsi->nofs = 1;
 		blkid_free_probe(probe);
 		return 1;
@@ -925,7 +904,7 @@ int fs_get_blkid(const char *pathname, struct fs_info *fsi)
 		strncpy(fsi->fstype, str, sizeof(fsi->fstype)-1);
 	else {
 		/* any difference from blkid_do_safeprobe rc=1? */
-		log_print("No file system type on %s.", pathname);
+		log_print_unless_silent("No file system type on %s.", pathname);
 		fsi->nofs = 1;
 		blkid_free_probe(probe);
 		return 1;
@@ -1212,9 +1191,8 @@ int wipe_known_signatures(struct cmd_context *cmd, struct device *dev,
 static int _snprintf_attr(char *buf, size_t buf_size, const char *sysfs_dir,
 			 const char *attribute, dev_t dev)
 {
-	if (dm_snprintf(buf, buf_size, "%sdev/block/%d:%d/%s", sysfs_dir,
-			(int)MAJOR(dev), (int)MINOR(dev),
-			attribute) < 0) {
+	if (dm_snprintf(buf, buf_size, "%sdev/block/%u:%u/%s", sysfs_dir,
+			MAJOR(dev), MINOR(dev),	attribute) < 0) {
 		log_warn("WARNING: sysfs path for %s attribute is too long.", attribute);
 		return 0;
 	}
