@@ -15,18 +15,41 @@
 #include "base/memory/zalloc.h"
 #include "lib/misc/lib.h"
 #include "lib/filters/filter.h"
+#include "lib/device/device_id.h"
 
 #ifdef __linux__
 
 #include <dirent.h>
 
+static int _lvmdevices_update_msg;
+
 static int _ignore_mpath_component(struct cmd_context *cmd, struct dev_filter *f, struct device *dev, const char *use_filter_name)
 {
+	dev_t mpath_devno = 0;
+
 	dev->filtered_flags &= ~DEV_FILTERED_MPATH_COMPONENT;
 
-	if (dev_is_mpath_component(cmd, dev)) {
+	if (dev_is_mpath_component(cmd, dev, &mpath_devno)) {
 		log_debug_devs("%s: Skipping mpath component device", dev_name(dev));
 		dev->filtered_flags |= DEV_FILTERED_MPATH_COMPONENT;
+
+		/*
+		 * Warn about misconfig where an mpath component is
+		 * in the devices file, but its mpath device is not.
+		 */
+		if ((dev->flags & DEV_MATCHED_USE_ID) && mpath_devno) {
+			if (!get_du_for_devno(cmd, mpath_devno)) {
+				struct device *mpath_dev = dev_cache_get_by_devt(cmd, mpath_devno);
+				log_warn("WARNING: devices file is missing %s (%u:%u) using multipath component %s.",
+					 mpath_dev ? dev_name(mpath_dev) : "unknown",
+					 MAJOR(mpath_devno), MINOR(mpath_devno), dev_name(dev));
+				if (!_lvmdevices_update_msg && strcmp(get_cmd_name(), "lvmdevices")) {
+					log_warn("See lvmdevices --update for devices file update.");
+					_lvmdevices_update_msg = 1;
+				}
+			}
+		}
+
 		return 0;
 	}
 
