@@ -55,15 +55,15 @@ static struct labeller_i *_alloc_li(const char *name, struct labeller *l)
 	struct labeller_i *li;
 	size_t len;
 
-	len = sizeof(*li) + strlen(name) + 1;
+	len = strlen(name);
 
-	if (!(li = malloc(len))) {
+	if (!(li = malloc(sizeof(*li) + len + 1))) {
 		log_error("Couldn't allocate memory for labeller list object.");
 		return NULL;
 	}
 
 	li->l = l;
-	strcpy(li->name, name);
+	memcpy(li->name, name, len + 1);
 
 	return li;
 }
@@ -253,7 +253,8 @@ struct label *label_create(struct labeller *labeller)
 
 
 /* global variable for accessing the bcache populated by label scan */
-struct bcache *scan_bcache;
+/* FIXME/TODO convert to cmd_context */
+static struct bcache *scan_bcache;
 
 #define BCACHE_BLOCK_SIZE_IN_SECTORS 256 /* 256*512 = 128K */
 
@@ -493,8 +494,8 @@ static int _scan_dev_open(struct device *dev)
 	 * dm_list_first() returns NULL if the list is empty.
 	 */
 	if (!(name_list = dm_list_first(&dev->aliases))) {
-		log_error("Device open %d:%d has no path names.",
-			  (int)MAJOR(dev->dev), (int)MINOR(dev->dev));
+		log_error("Device open %u:%u has no path names.",
+			  MAJOR(dev->dev), MINOR(dev->dev));
 		return 0;
 	}
 	name_sl = dm_list_item(name_list, struct dm_str_list);
@@ -534,8 +535,8 @@ static int _scan_dev_open(struct device *dev)
 			 * drop any other invalid aliases before retrying open with
 			 * any remaining valid paths.
 			 */
-			log_debug("Drop alias for %d:%d failed open %s (%d)",
-				  (int)MAJOR(dev->dev), (int)MINOR(dev->dev), name, errno);
+			log_debug("Drop alias for %u:%u failed open %s (%d).",
+				  MAJOR(dev->dev), MINOR(dev->dev), name, errno);
 			dev_cache_failed_path(dev, name);
 			dev_cache_verify_aliases(dev);
 			goto next_name;
@@ -544,8 +545,8 @@ static int _scan_dev_open(struct device *dev)
 
 	/* Verify that major:minor from the path still match dev. */
 	if ((fstat(fd, &sbuf) < 0) || (sbuf.st_rdev != dev->dev)) {
-		log_warn("Invalid path %s for device %d:%d, trying different path.",
-			 name, (int)MAJOR(dev->dev), (int)MINOR(dev->dev));
+		log_warn("Invalid path %s for device %u:%u, trying different path.",
+			 name, MAJOR(dev->dev), MINOR(dev->dev));
 		(void)close(fd);
 		dev_cache_failed_path(dev, name);
 		dev_cache_verify_aliases(dev);
@@ -659,8 +660,8 @@ static int _scan_list(struct cmd_context *cmd, struct dev_filter *f,
 
 		if (!_in_bcache(devl->dev)) {
 			if (!_scan_dev_open(devl->dev)) {
-				log_debug_devs("Scan failed to open %d:%d %s.",
-					       (int)MAJOR(devl->dev->dev), (int)MINOR(devl->dev->dev), dev_name(devl->dev));
+				log_debug_devs("Scan failed to open %u:%u %s.",
+					       MAJOR(devl->dev->dev), MINOR(devl->dev->dev), dev_name(devl->dev));
 				dm_list_del(&devl->list);
 				devl->dev->flags |= DEV_SCAN_NOT_READ;
 				continue;
@@ -710,10 +711,9 @@ static int _scan_list(struct cmd_context *cmd, struct dev_filter *f,
 			 */
 			bcache_put(bb);
 
-			log_debug_devs("Processing data from device %s %d:%d di %d",
+			log_debug_devs("Processing data from device %s %u:%u di %d.",
 				       dev_name(devl->dev),
-				       (int)MAJOR(devl->dev->dev),
-				       (int)MINOR(devl->dev->dev),
+				       MAJOR(devl->dev->dev), MINOR(devl->dev->dev),
 				       devl->dev->bcache_di);
 
 			ret = _process_block(cmd, f, devl->dev, headers_buf, sizeof(headers_buf), 0, 0, &is_lvm_device);
@@ -769,13 +769,16 @@ static int _scan_list(struct cmd_context *cmd, struct dev_filter *f,
 #define MIN_BCACHE_BLOCKS 32    /* 4MB (32 * 128KB) */
 #define MAX_BCACHE_BLOCKS 4096  /* 512MB (4096 * 128KB) */
 
-static int _setup_bcache(void)
+int label_scan_setup_bcache(void)
 {
 	struct io_engine *ioe = NULL;
 	int iomem_kb = io_memory_size();
 	int block_size_kb = (BCACHE_BLOCK_SIZE_IN_SECTORS * 512) / 1024;
 	int cache_blocks;
 
+	if (scan_bcache)
+		return 1;
+
 	cache_blocks = iomem_kb / block_size_kb;
 
 	if (cache_blocks < MIN_BCACHE_BLOCKS)
@@ -872,7 +875,6 @@ int label_scan_for_pvid(struct cmd_context *cmd, char *pvid, struct device **dev
 	char buf[LABEL_SIZE] __attribute__((aligned(8)));
 	struct dm_list devs;
 	struct dev_iter *iter;
-	struct device_list *devl, *devl2;
 	struct device *dev;
 	struct pv_header *pvh;
 	int ret = 0;
@@ -888,6 +890,9 @@ int label_scan_for_pvid(struct cmd_context *cmd, char *pvid, struct device **dev
 		return 0;
 	}
 
+	if (!label_scan_setup_bcache())
+		return_0;
+
 	/*
 	 * Iterating over all available devices with cmd->filter filters
 	 * devices; those returned from dev_iter_get are the devs that
@@ -899,25 +904,10 @@ int label_scan_for_pvid(struct cmd_context *cmd, char *pvid, struct device **dev
 		return 0;
 	}
 
-	log_debug_devs("Filtering devices to scan");
-
-	while ((dev = dev_iter_get(cmd, iter))) {
-		if (!(devl = zalloc(sizeof(*devl))))
-			continue;
-		devl->dev = dev;
-		dm_list_add(&devs, &devl->list);
-	};
-	dev_iter_destroy(iter);
-
-	if (!scan_bcache) {
-		if (!_setup_bcache())
-			goto_out;
-	}
 
 	log_debug_devs("Reading labels for pvid");
 
-	dm_list_iterate_items(devl, &devs) {
-		dev = devl->dev;
+	while ((dev = dev_iter_get(cmd, iter))) {
 
 		memset(buf, 0, sizeof(buf));
 
@@ -932,7 +922,7 @@ int label_scan_for_pvid(struct cmd_context *cmd, char *pvid, struct device **dev
 		pvh = (struct pv_header *)(buf + 32);
 
 		if (!memcmp(pvh->pv_uuid, pvid, ID_LEN)) {
-			*dev_out = devl->dev;
+			*dev_out = dev;
 			_scan_dev_close(dev);
 			break;
 		}
@@ -941,10 +931,8 @@ int label_scan_for_pvid(struct cmd_context *cmd, char *pvid, struct device **dev
 	}
 	ret = 1;
  out:
-	dm_list_iterate_items_safe(devl, devl2, &devs) {
-		dm_list_del(&devl->list);
-		free(devl);
-	}
+	dev_iter_destroy(iter);
+
 	return ret;
 }
 
@@ -1034,8 +1022,8 @@ int label_scan_vg_online(struct cmd_context *cmd, const char *vgname,
 	 */
 	dm_list_iterate_items(po, &pvs_online) {
 		if (!(po->dev = setup_dev_in_dev_cache(cmd, po->devno, po->devname[0] ? po->devname : NULL))) {
-			log_debug("No device found for quick mapping of online PV %d:%d %s PVID %s",
-				  (int)MAJOR(po->devno), (int)MINOR(po->devno), po->devname, po->pvid);
+			log_debug("No device found for quick mapping of online PV %u:%u %s PVID %s.",
+				  MAJOR(po->devno), MINOR(po->devno), po->devname, po->pvid);
 			try_dev_scan = 1;
 			continue;
 		}
@@ -1059,8 +1047,8 @@ int label_scan_vg_online(struct cmd_context *cmd, const char *vgname,
 			if (po->dev)
 				continue;
 			if (!(po->dev = dev_cache_get_by_devt(cmd, po->devno))) {
-				log_error("No device found for %d:%d PVID %s",
-					  (int)MAJOR(po->devno), (int)MINOR(po->devno), po->pvid);
+				log_error("No device found for %u:%u PVID %s.",
+					  MAJOR(po->devno), MINOR(po->devno), po->pvid);
 				goto bad;
 			}
 			if (!(devl = dm_pool_zalloc(cmd->mem, sizeof(*devl))))
@@ -1266,10 +1254,17 @@ int label_scan(struct cmd_context *cmd)
 	dm_list_init(&scan_devs);
 	dm_list_init(&hints_list);
 
-	if (!scan_bcache) {
-		if (!_setup_bcache())
-			return_0;
-	}
+	if (!label_scan_setup_bcache())
+		return_0;
+
+	/*
+	 * Initialize cache of dm device uuids, which uses a special dm kernel
+	 * feature for efficiently querying many dm devs together.  (It's done
+	 * here, before processing the hints file, so that the dm uuid checks
+	 * in hint processing can benefit from the dm uuid cache.)
+	 */
+	if (!dm_devs_cache_update())
+		return_0;
 
 	/*
 	 * Creates a list of available devices, does not open or read any,
@@ -1505,12 +1500,10 @@ int label_scan(struct cmd_context *cmd)
  */
 int label_read_pvid(struct device *dev, int *has_pvid)
 {
-	char buf[4096] __attribute__((aligned(8)));
+	char buf[4096] __attribute__((aligned(8))) = { 0 };
 	struct label_header *lh;
 	struct pv_header *pvh;
 
-	memset(buf, 0, sizeof(buf));
-
 	if (!label_scan_open(dev))
 		return_0;
 
@@ -1580,10 +1573,8 @@ int label_scan_devs(struct cmd_context *cmd, struct dev_filter *f, struct dm_lis
 {
 	struct device_list *devl;
 
-	if (!scan_bcache) {
-		if (!_setup_bcache())
-			return 0;
-	}
+	if (!label_scan_setup_bcache())
+		return_0;
 
 	dm_list_iterate_items(devl, devs) {
 		if (_in_bcache(devl->dev))
@@ -1599,10 +1590,8 @@ int label_scan_devs_rw(struct cmd_context *cmd, struct dev_filter *f, struct dm_
 {
 	struct device_list *devl;
 
-	if (!scan_bcache) {
-		if (!_setup_bcache())
-			return 0;
-	}
+	if (!label_scan_setup_bcache())
+		return_0;
 
 	dm_list_iterate_items(devl, devs) {
 		if (_in_bcache(devl->dev))
@@ -1657,6 +1646,9 @@ void label_scan_invalidate_lv(struct cmd_context *cmd, struct logical_volume *lv
 	struct device *dev;
 	dev_t devt;
 
+	/* FIXME: use dev_cache_get_existing() with the lv name,
+	   which allow us to skip the getting devno from lv_info. */
+
 	if (lv_info(cmd, lv, 0, &lvinfo, 0, 0) && lvinfo.exists) {
 		/* FIXME: Still unclear what is it supposed to find */
 		devt = MKDEV(lvinfo.major, lvinfo.minor);
@@ -1667,12 +1659,7 @@ void label_scan_invalidate_lv(struct cmd_context *cmd, struct logical_volume *lv
 
 void label_scan_invalidate_lvs(struct cmd_context *cmd, struct dm_list *lvs)
 {
-	struct dm_list *devs;
-	struct dm_active_device *dm_dev;
-	unsigned devs_features = 0;
-	struct device *dev;
 	struct lv_list *lvl;
-	dev_t devt;
 
 	/*
 	 * This is only needed when the command sees PVs stacked on LVs which
@@ -1680,32 +1667,15 @@ void label_scan_invalidate_lvs(struct cmd_context *cmd, struct dm_list *lvs)
 	 */
 	if (!cmd->scan_lvs)
 		return;
-	log_debug("invalidating devs for any pvs on lvs");
 
-	if (get_device_list(NULL, &devs, &devs_features)) {
-		if (devs_features & DM_DEVICE_LIST_HAS_UUID) {
-			dm_list_iterate_items(dm_dev, devs)
-				if (dm_dev->uuid &&
-				    strncmp(dm_dev->uuid, UUID_PREFIX, sizeof(UUID_PREFIX) - 1) == 0) {
-					devt = MKDEV(dm_dev->major, dm_dev->minor);
-					if ((dev = dev_cache_get_by_devt(cmd, devt)))
-						label_scan_invalidate(dev);
-				}
-			/* ATM no further caching for any lvconvert command
-			 * TODO: any other command to be skipped ??
-			 */
-			if (strcmp(cmd->name, "lvconvert")) {
-				dm_device_list_destroy(&cmd->cache_dm_devs);
-				cmd->cache_dm_devs = devs; /* cache to avoid unneeded checks */
-				devs = NULL;
-			}
-		}
-		dm_device_list_destroy(&devs);
-	}
+	log_debug("Invalidating devs for any PVs on LVs.");
 
-	if (!(devs_features & DM_DEVICE_LIST_HAS_UUID))
+	if (dm_devs_cache_use())
+		dm_devs_cache_label_invalidate(cmd);
+	else {
 		dm_list_iterate_items(lvl, lvs)
 			label_scan_invalidate_lv(cmd, lvl->lv);
+	}
 }
 
 /*
@@ -1776,16 +1746,6 @@ int label_scan_dev(struct cmd_context *cmd, struct device *dev)
 	return 1;
 }
 
-int label_scan_setup_bcache(void)
-{
-	if (!scan_bcache) {
-		if (!_setup_bcache())
-			return 0;
-	}
-
-	return 1;
-}
-
 /*
  * This is needed to write to a new non-lvm device.
  * Scanning that dev would not keep it open or in
@@ -1831,15 +1791,15 @@ int label_scan_reopen_rw(struct device *dev)
 	int fd;
 
 	if (dm_list_empty(&dev->aliases)) {
-		log_error("Cannot reopen rw device %d:%d with no valid paths di %d fd %d.",
-			  (int)MAJOR(dev->dev), (int)MINOR(dev->dev), dev->bcache_di, dev->bcache_fd);
+		log_error("Cannot reopen rw device %u:%u with no valid paths di %d fd %d.",
+			  MAJOR(dev->dev), MINOR(dev->dev), dev->bcache_di, dev->bcache_fd);
 		return 0;
 	}
 
 	name = dev_name(dev);
 	if (!name || name[0] != '/') {
-		log_error("Cannot reopen rw device %d:%d with no valid name di %d fd %d.",
-			  (int)MAJOR(dev->dev), (int)MINOR(dev->dev), dev->bcache_di, dev->bcache_fd);
+		log_error("Cannot reopen rw device %u:%u with no valid name di %d fd %d.",
+			  MAJOR(dev->dev), MINOR(dev->dev), dev->bcache_di, dev->bcache_fd);
 		return 0;
 	}
 
