@@ -9,18 +9,18 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "lib.h"
+#include "lib/misc/lib.h"
 
-#include "daemon-io.h"
-#include "lvmpolld-client.h"
-#include "lvmpolld-protocol.h"
-#include "metadata-exported.h"
-#include "polldaemon.h"
-#include "toolcontext.h"
-#include "lvm2cmd.h"
+#include "libdaemon/client/daemon-io.h"
+#include "lib/lvmpolld/lvmpolld-client.h"
+#include "daemons/lvmpolld/lvmpolld-protocol.h"
+#include "lib/metadata/metadata-exported.h"
+#include "lib/lvmpolld/polldaemon.h"
+#include "lib/commands/toolcontext.h"
+#include "tools/lvm2cmd.h"
 
 struct progress_info {
 	unsigned error:1;
@@ -207,7 +207,7 @@ static int _process_poll_init(const struct cmd_context *cmd, const char *poll_ty
 	const char *e = getenv("LVM_SYSTEM_DIR");
 	int r = 0; 
 
-	str = dm_malloc(INTERV_SIZE * sizeof(char));
+	str = malloc(INTERV_SIZE * sizeof(char));
 	if (!str)
 		return r;
 
@@ -247,6 +247,13 @@ static int _process_poll_init(const struct cmd_context *cmd, const char *poll_ty
 		goto out_req;
 	}
 
+	if (parms->devicesfile[0] &&
+	    !(daemon_request_extend(req, LVMPD_PARM_DEVICESFILE " = %s",
+			            parms->devicesfile, NULL))) {
+		log_error("Failed to create %s request." , poll_type);
+		goto out_req;
+	}
+
 	rep = daemon_send(_lvmpolld, req);
 
 	if (rep.error) {
@@ -266,7 +273,7 @@ out_rep:
 	daemon_reply_destroy(rep);
 out_req:
 	daemon_request_destroy(req);
-	dm_free(str);
+	free(str);
 
 	return r;
 }
@@ -277,17 +284,17 @@ int lvmpolld_poll_init(const struct cmd_context *cmd, const struct poll_operatio
 	int r = 0;
 
 	if (!id->uuid) {
-		log_error(INTERNAL_ERROR "Use of lvmpolld requires uuid set");
+		log_error(INTERNAL_ERROR "Use of lvmpolld requires uuid set.");
 		return 0;
 	}
 
 	if (!id->vg_name) {
-		log_error(INTERNAL_ERROR "Use of lvmpolld requires vgname set");
+		log_error(INTERNAL_ERROR "Use of lvmpolld requires vgname set.");
 		return 0;
 	}
 
 	if (!id->lv_name) {
-		log_error(INTERNAL_ERROR "Use of lvmpolld requires lvname set");
+		log_error(INTERNAL_ERROR "Use of lvmpolld requires lvname set.");
 		return 0;
 	}
 
@@ -314,7 +321,7 @@ int lvmpolld_poll_init(const struct cmd_context *cmd, const struct poll_operatio
 			log_error(INTERNAL_ERROR "Unsupported poll operation.");
 		}
 	} else
-		log_error(INTERNAL_ERROR "Unsupported poll operation");
+		log_error(INTERNAL_ERROR "Unsupported poll operation.");
 
 	return r;
 }
@@ -327,7 +334,7 @@ int lvmpolld_request_info(const struct poll_operation_id *id, const struct daemo
 	*finished = 1;
 
 	if (!id->uuid) {
-		log_error(INTERNAL_ERROR "use of lvmpolld requires uuid being set");
+		log_error(INTERNAL_ERROR "Use of lvmpolld requires uuid being set.");
 		return 0;
 	}
 
