@@ -433,12 +433,14 @@ struct pv_segment {
  */
 
 struct format_instance {
-	unsigned ref_count;	/* Refs to this fid from VG and PV structs */
 	struct dm_pool *mem;
 
-	uint32_t type;
 	const struct format_type *fmt;
 
+	unsigned ref_count;	/* Refs to this fid from VG and PV structs */
+
+	uint32_t type;
+
 	/*
 	 * Each mda in a vg is on exactly one of the below lists.
 	 * MDAs on the 'in_use' list will be read from / written to
@@ -697,7 +699,6 @@ struct lvresize_params {
 	int extend_fs_error;    /* FS extend error after LV extend success */
 	int vg_changed_error;   /* VG metadata was modified during fs resize */
 
-	const char *lockopt;
 	char *lockd_lv_refresh_path; /* set during resize to use for refresh at the end */
 	char *lockd_lv_refresh_uuid; /* set during resize to use for refresh at the end */
 
@@ -1467,6 +1468,10 @@ struct vgcreate_params {
 	const char *lock_args;
 };
 
+struct lvremove_params {
+	struct dm_list removed_uuids; /* entries are str_list */
+};
+
 int validate_major_minor(const struct cmd_context *cmd,
 			 const struct format_type *fmt,
 			 int32_t major, int32_t minor);
