@@ -32,7 +32,6 @@
 //#define STRIPE_SIZE_MIN ( (unsigned) lvm_getpagesize() >> SECTOR_SHIFT)	/* PAGESIZE in sectors */
 //#define STRIPE_SIZE_MAX ( 512L * 1024L >> SECTOR_SHIFT)	/* 512 KB in sectors */
 //#define STRIPE_SIZE_LIMIT ((UINT_MAX >> 2) + 1)
-//#define MAX_RESTRICTED_LVS 255	/* Used by FMT_RESTRICTED_LVIDS */
 #define MIN_PE_SIZE     (8192L >> SECTOR_SHIFT) /* 8 KB in sectors - format1 only */
 #define MAX_PE_SIZE     (16L * 1024L * (1024L >> SECTOR_SHIFT) * 1024L) /* format1 only */
 #define MIRROR_LOG_OFFSET	2	/* sectors */
@@ -74,7 +73,6 @@ struct cached_vg_fmtdata;
 
 /* Per-format per-metadata area operations */
 struct metadata_area_ops {
-	struct dm_list list;
 	struct volume_group *(*vg_read) (struct cmd_context *cmd,
 					 struct format_instance * fi,
 					 const char *vg_name,
@@ -184,7 +182,7 @@ struct metadata_area_ops {
 
 struct metadata_area {
 	struct dm_list list;
-	struct metadata_area_ops *ops;
+	const struct metadata_area_ops *ops;
 	void *metadata_locn;
 	uint32_t status;
 	uint64_t header_start; /* mda_header.start */
