@@ -322,12 +322,28 @@ prepare_lvmdbusd() {
 	"$daemon" $lvmdbusdebug > debug.log_LVMDBUSD_out 2>&1 &
 	local pid=$!
 
-	sleep 1
 	echo -n "## checking lvmdbusd IS running..."
+	if which dbus-send &>/dev/null ; then
+	for i in {100..0}; do
+		dbus-send --system --dest=org.freedesktop.DBus --type=method_call --print-reply /org/freedesktop/DBus org.freedesktop.DBus.ListNames > dbus_services
+		grep -q com.redhat.lvmdbus1 dbus_services && break
+		sleep .1
+	done
+	else
+		sleep 2
+	fi
+
+	if [ "$i" -eq 0 ] ; then
+		printf "\nFailed to serve lvm dBus service in 10 seconds.\n"
+		sed -e "s,^,## DBUS_SERVICES: ," dbus_services
+		ps aux
+		return 1
+	fi
+
 	comm=
 	# TODO: Is there a better check than wait 1 second and check pid?
 	if ! comm=$(ps -p $pid -o comm=) >/dev/null || [[ $comm != lvmdbusd ]]; then
-		echo "Failed to start lvmdbusd daemon"
+		printf "\nFailed to start lvmdbusd daemon\n"
 		return 1
 	fi
 	echo "$pid" > LOCAL_LVMDBUSD
@@ -696,7 +712,7 @@ prepare_ramdisk() {
 }
 
 prepare_real_devs() {
-	aux lvmconf 'devices/scan = "/dev"'
+	lvmconf 'devices/scan = "/dev"'
 
 	touch REAL_DEVICES
 
@@ -705,7 +721,7 @@ prepare_real_devs() {
 		while read path; do
 			REAL_DEVICES[count]=$path
 			count=$((  count + 1 ))
-			aux extend_filter "a|$path|"
+			extend_filter "a|$path|"
 			dd if=/dev/zero of="$path" bs=32k count=1
 			wipefs -a "$path" 2>/dev/null || true
 		done < "$LVM_TEST_DEVICE_LIST"
@@ -738,6 +754,7 @@ prepare_scsi_debug_dev() {
 
 	for i in {1..20} ; do
 		sleep .1 # allow for async Linux SCSI device registration
+		ls /sys/block/sd*/device/model >/dev/null 2>&1 || continue
 		DEBUG_DEV="/dev/$(grep -H scsi_debug /sys/block/sd*/device/model | cut -f4 -d /)"
 		test -b "$DEBUG_DEV" && break
 	done
@@ -1035,7 +1052,7 @@ prepare_devs() {
 
 	touch DEVICES
 	prepare_backing_dev $(( n * devsize + 2 * header_shift ))
-	blkdiscard "$BACKING_DEV" 2>/dev/null || true
+	test -e NO_BLKDISCARD_Z || { blkdiscard "$BACKING_DEV" 2>/dev/null || true; }
 	echo -n "## preparing $n devices..."
 
 	local size=$(( devsize * 2048 )) # sectors
@@ -1548,8 +1565,9 @@ global/cache_dump_executable = "$LVM_TEST_CACHE_DUMP_CMD"
 global/cache_repair_executable = "$LVM_TEST_CACHE_REPAIR_CMD"
 global/cache_restore_executable = "$LVM_TEST_CACHE_RESTORE_CMD"
 global/detect_internal_vg_cache_corruption = 1
-global/fallback_to_local_locking = 0
 global/etc = "$LVM_SYSTEM_DIR"
+global/event_activation = 1
+global/fallback_to_local_locking = 0
 global/locking_type=$LVM_TEST_LOCKING
 global/notify_dbus = 0
 global/si_unit_consistency = 1
@@ -1557,8 +1575,8 @@ global/thin_check_executable = "$LVM_TEST_THIN_CHECK_CMD"
 global/thin_dump_executable = "$LVM_TEST_THIN_DUMP_CMD"
 global/thin_repair_executable = "$LVM_TEST_THIN_REPAIR_CMD"
 global/thin_restore_executable = "$LVM_TEST_THIN_RESTORE_CMD"
-global/use_lvmpolld = $LVM_TEST_LVMPOLLD
 global/use_lvmlockd = $LVM_TEST_LVMLOCKD
+global/use_lvmpolld = $LVM_TEST_LVMPOLLD
 log/activation = 1
 log/file = "$TESTDIR/debug.log"
 log/indent = 1
@@ -1828,10 +1846,11 @@ version_at_least() {
 # i.e.   dm_target_at_least  dm-thin-pool  1 0
 target_at_least() {
 	rm -f debug.log strace.log
-	case "$1" in
-	  dm-vdo) modprobe "kvdo" || true ;;
-	  dm-*) modprobe "$1" || true ;;
-	esac
+	modprobe "$1" || {
+		case "$1" in
+		  dm-vdo) modprobe "kvdo" || true ;;
+		esac
+	}
 
 	if test "$1" = dm-raid; then
 		case "$(uname -r)" in
@@ -1895,6 +1914,12 @@ have_vdo() {
 		return 1
 	}
 	target_at_least dm-vdo "$@"
+
+	vdoformat=$(lvm lvmconfig --typeconfig full --valuesonly global/vdo_format_executable || true)
+	# Remove surrouding "" around string
+	# TODO: lvmconfig should have an option to give this output directly
+	vdoformat=${vdoformat//\"}
+	test -x "$vdoformat" || { echo "No executable to format VDO \"$vdoformat\"..."; return 1; }
 }
 
 have_writecache() {
