@@ -717,9 +717,13 @@ struct KMsg : Source {
         unsigned long t;
         time_t tt;
         size_t len;
+        const char *delimiter = strchr(buf, ';');
 
         buf[ *sz ] = 0;
-        if (sscanf( buf, "%u,%u,%lu,-;%n", &level, &num, &t, &pos ) == 3) {
+
+        if ( sscanf( buf, "%u,%u,%lu,-%n", &level, &num, &t, &pos) == 3 ) {
+            if ( delimiter++ && ( delimiter - buf ) > pos )
+                pos = delimiter - buf;
             memcpy( newbuf, buf, *sz );
             tt = time( 0 );
             len = snprintf( buf, 64, "[%lu.%06lu] <%u> ", t / 1000000, t % 1000000, level );
