@@ -61,6 +61,7 @@
 #define ARG_COUNTABLE 0x00000001	/* E.g. -vvvv */
 #define ARG_GROUPABLE 0x00000002	/* E.g. --addtag */
 #define ARG_NONINTERACTIVE 0x00000004	/* only for use in noninteractive mode  */
+#define ARG_LONG_OPT  0x00000008	/* arg has long format option  */
 
 struct arg_values {
 	char *value;
@@ -164,7 +165,7 @@ int mirror_remove_missing(struct cmd_context *cmd,
 
 
 int vgchange_activate(struct cmd_context *cmd, struct volume_group *vg,
-		       activation_change_t activate, int vg_complete_to_activate);
+		       activation_change_t activate, int vg_complete_to_activate, char *root_dm_uuid);
 
 int vgchange_background_polling(struct cmd_context *cmd, struct volume_group *vg);
 
