@@ -14,6 +14,7 @@
  */
 
 #include "tools.h"
+#include "lib/device/online.h"
 
 static int _vgremove_single(struct cmd_context *cmd, const char *vg_name,
 			    struct volume_group *vg,
@@ -42,9 +43,6 @@ static int _vgremove_single(struct cmd_context *cmd, const char *vg_name,
 	unsigned lv_count, missing;
 	int ret;
 
-	if (!vg_check_status(vg, EXPORTED_VG))
-		return_ECMD_FAILED;
-
 	lv_count = vg_visible_lvs(vg);
 
 	if (lv_count) {
@@ -68,12 +66,18 @@ static int _vgremove_single(struct cmd_context *cmd, const char *vg_name,
 		}
 	}
 
-	if (!lockd_free_vg_before(cmd, vg, 0))
+	if (vg->pool_metadata_spare_lv &&
+	    !lvremove_single(cmd, vg->pool_metadata_spare_lv, &void_handle))
+		return_ECMD_FAILED;
+
+	if (!lockd_free_vg_before(cmd, vg, 0, arg_is_set(cmd, yes_ARG)))
 		return_ECMD_FAILED;
 
 	if (!force && !vg_remove_check(vg))
 		return_ECMD_FAILED;
 
+	online_vgremove(vg);
+
 	vg_remove_pvs(vg);
 
 	if (!vg_remove(vg))
